package dk.nsi.encoder;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;

public class Encoder extends Task {

    private File destinationDir = null; // the destination directory
    private File sourceDir = null;
    private String sourceDirName;

    public void setSourceDir(File sourceDir) {
        this.sourceDir = sourceDir;
    }

    public void setDestinationDir(File destinationDir) {
        this.destinationDir = destinationDir;
    }

    @Override
    public void execute() throws BuildException {
        sourceDirName = sourceDir.getAbsolutePath();

        List<File> files = scan(sourceDir);

        for (File file : files) {
            File destinationFile = convert(file);
            destinationFile.getParentFile().mkdirs(); // Ensure folder structure

            try {
                copyFile(file, destinationFile);
            } catch (IOException e) {
                throw new BuildException(e);
            }
        }
    }

    private void copyFile(File file, File destinationFile) throws IOException {
        FileInputStream fis = new FileInputStream(file);
        byte[] contents = new byte[fis.available()];
        fis.read(contents, 0, contents.length);

        String asString = new String(contents, "ISO8859_1");

        BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(destinationFile), "UTF8"));
        out.write(asString);
        out.flush();
        out.close();
    }

    private File convert(File file) {
        String tmp = file.getAbsolutePath();
        String rest = tmp.substring(sourceDirName.length());

        return new File(destinationDir, rest);
    }

    private List<File> scan(File dir) {
        List<File> res = new ArrayList<File>();
        File[] files = dir.listFiles();
        for (int i = 0; i < files.length; i++) {
            File file = files[i];
            if(file.isDirectory()) {
                res.addAll(scan(file));
            } else {
                res.add(file);
            }
        }
        return res;
    }
}