package dk.rsd.dcc;

import dk.nsp.test.idp.builder.Builders;
import dk.nsp.test.idp.builder.SecurityTokenServiceBuilder;
import dk.nsp.test.idp.model.EmployeeIdentity;
import dk.nsp.test.idp.vault.RemoteCredentialVault;
import dk.rsd.dcc.configuration.IntegrationTestConfiguration;
import dk.rsd.dcc.util.WebServiceException;
import dk.rsd.dcc.util.WsTestClient;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.gw._2007_09_01.SosiGWFacade;
import dk.sosi.seal.xml.XmlUtil;
import dk.sundhedsdatastyrelsen.dcc.xsd.DccRequestType;
import oasis.names.tc.saml._2_0.assertion.AssertionType;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.oasis_open.docs.wss._2004._01.oasis_200401_wss_wssecurity_secext_1_0.Security;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;
import org.w3c.dom.Node;
import sun.reflect.generics.reflectiveObjects.NotImplementedException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.UUID;

import static org.junit.Assert.assertNotNull;

@ContextConfiguration(classes = {IntegrationTestConfiguration.class})
@RunWith(SpringRunner.class)
public class DccSosiGatewayTest {
    @Value("${dcc.host}")
    private String dccHost;

    @Value("${nts.host}")
    private String ntsHost;  //"http://test1.ekstern-test.nspop.dk:8080";

    @Value("${sosigw.host}")
    private String sosigwHost; // "http://test1.ekstern-test.nspop.dk:8080";

    private boolean partition = true;

    private String keystorePassword = "Test1234" ;

    private SosiGWFacadeTestClient sosiGWFacade;

    private EmployeeIdentity sosiGwIdentity;

    @Before
    public void setup() throws Exception {

        SecurityTokenServiceBuilder sts = Builders.securityTokenService().at(URI.create(sosigwHost + "/sosigw/service/sosigw/createIdCardFromBST"));
        sosiGwIdentity =  Builders.employeeIdentity().representing(Builders.employee().identifiedBy(UUID.fromString("53767053-0628-4176-b66f-0da3a0b6e868")).withNationalRole("urn:dk:healthcare:national-federation-role:code:41006:value:TransplantationskoordinatorR6")).employedAt(Builders.organization().named("Sundhedsdatastyrelsen").identifiedBy("33257872")).usedBy(Builders.serviceConsumer().named("Service Consumer Test").identifiedBy(new RemoteCredentialVault(URI.create("https://www.nspop.dk/download/attachments/190481050/NSP_Test_Service_Consumer_sds.p12"), "Test1234".toCharArray(), true))).verifiedBy(Builders.identityProvider().named("https://idp.test.nspop.dk").identifiedBy(new RemoteCredentialVault(URI.create("https://www.nspop.dk/download/attachments/190481050/NSP_Test_Identity_Provider_sds.p12"), "Test1234".toCharArray(), true))).using(sts).build();
        sosiGWFacade = getSosiGWFacadePort(sosiGwIdentity);
        sosiGWFacade.getValidIdCard(null, "Ignored");
    }
    @After
    public void tearDown() throws Exception {
        sosiGWFacade.close();
    }

    @Test
    public void testCallDccToSosiGwToNTS() throws Exception {

        System.out.println("Getting NTS Port");
        String proxyEndpoint = dccHost + "/decoupling";
        System.out.println("Calling NTS through DCC and SOSI GW. In that order.");
        DccRequestType dccRequest = new DccRequestType();
        dccRequest.setDccRequstUrl("payload");
        String resultat = sosiGWFacade.callDcc(dccRequest, proxyEndpoint, "http://nspop.dk/nts/2013/05#verifysosigw");

        System.out.println("resultat " + resultat);
        assertNotNull( resultat);
        System.out.println("Done");
    }

    private SosiGWFacadeTestClient getSosiGWFacadePort(EmployeeIdentity employeeIdentity) throws JAXBException, URISyntaxException {

        return new SosiGWFacadeTestClient(sosigwHost + "/sosigw/service/sosigw", employeeIdentity);
    }

    public static class SosiGWFacadeTestClient implements SosiGWFacade, AutoCloseable {

        private final WsTestClient nspSoapClient;

        private final String endpoint;

        private final EmployeeIdentity employeeIdentity;

        private final JAXBContext jaxbContext;

        public SosiGWFacadeTestClient(String endpoint, EmployeeIdentity employeeIdentity) throws JAXBException, URISyntaxException {
            this(endpoint, JAXBContext.newInstance(DccRequestType.class, RequestIdCardDigestForSigningResponse.class, SignIdCardRequestBody.class), employeeIdentity);
        }

        public SosiGWFacadeTestClient(String endpoint, JAXBContext jaxbContext, EmployeeIdentity employeeIdentity) throws JAXBException, URISyntaxException {
            this.nspSoapClient = new WsTestClient(jaxbContext, null, employeeIdentity);
            this.endpoint = endpoint;
            this.employeeIdentity = employeeIdentity;
            this.jaxbContext = jaxbContext;
        }

        public String callDcc(DccRequestType dccRequest, String endpoint, String soapAction) throws WebServiceException {
            Node body = nspSoapClient.callService(endpoint, DccRequestType.class, "http://www.sundhedsdatastyrelsen.dk/dcc/xsd", "dccRequest", dccRequest, soapAction);
            Node ok = body.getOwnerDocument().getElementsByTagName("OK").item(0);
            String result = ok.getTextContent();
            return result;
        }


        @Override
        public AssertionType getValidIdCard(Security wsseHeader, String getValidIdCardRequestBody) throws dk.sosi.gw._2007_09_01.DGWSFault {
            Node body = callSosiNode(String.class, "http://sosi.dk/gw/2007.09.01", "getValidIdCardRequestBody", getValidIdCardRequestBody, "http://sosi.dk/gw/2007.09.01#getValidIdCard");
            String responseBody = XmlUtil.node2String(body);
            String employeeeNationalRole = employeeIdentity.getEmployee().getNationalRole();
            Assert.assertTrue(responseBody.contains(employeeeNationalRole));
            return null;
        }

        @Override
        public String signIdCard(Security wsseHeader, SignIdCardRequestBody signIdCardRequestBody) throws dk.sosi.gw._2007_09_01.DGWSFault {
            throw new NotImplementedException();
        }


        @Override
        public void logout(Security wsseHeader, String logoutRequestBody) {
            try {
                callSosi(String.class, "http://sosi.dk/gw/2007.09.01", "", logoutRequestBody, "http://sosi.dk/gw/2007.09.01#logout");
            } catch (dk.sosi.gw._2007_09_01.DGWSFault e) {
                throw new RuntimeException(e);
            }
        }

        @Override
        public RequestIdCardDigestForSigningResponse requestIdCardDigestForSigning(Security wsseHeader, String requestIdCardDigestForSigningRequestBody) throws dk.sosi.gw._2007_09_01.DGWSFault {
            throw new NotImplementedException();
        }

        @Override
        public String logoutWithResponse(Security wsseHeader, String logoutRequestBody) throws dk.sosi.gw._2007_09_01.DGWSFault {
            return callSosi(String.class, "http://sosi.dk/gw/2007.09.01", "", logoutRequestBody, "http://sosi.dk/gw/2014.06.01#logoutWithResponse");
        }


        private <RequestType> Node callSosiNode(Class<RequestType> requestClazz, String requestNamespace, String requestLocalPart, RequestType request, String soapAction) throws dk.sosi.gw._2007_09_01.DGWSFault {
            try {
                Node body = nspSoapClient.callService(endpoint, requestClazz, requestNamespace, requestLocalPart, request, soapAction);
                return body;
            } catch (WebServiceException e) {
                throw new dk.sosi.gw._2007_09_01.DGWSFault(e.getFaultCode(), e.getMessage());
            }
        }

        public <RequestType, ResponseType> ResponseType callSosi(Class<RequestType> requestClazz, String requestNamespace, String requestLocalPart, RequestType request, String soapAction) throws dk.sosi.gw._2007_09_01.DGWSFault {
            try {
                Node body = nspSoapClient.callService(endpoint, requestClazz, requestNamespace, requestLocalPart, request, soapAction);
                return (ResponseType) jaxbContext.createUnmarshaller().unmarshal(body.getFirstChild());
            } catch (JAXBException e) {
                throw new RuntimeException(e);
            } catch (WebServiceException e) {
                throw new dk.sosi.gw._2007_09_01.DGWSFault(e.getFaultCode(), e.getMessage());
            }

        }

        @Override
        public void close() throws Exception {
            nspSoapClient.close();
        }
    }
}
