package dk.rsd.dcc.util;

import dk.nsp.test.idp.EmployeeIdentities;
import dk.nsp.test.idp.model.Identity;
import dk.sundhedsdatastyrelsen.dcc.wsdl.DGWSFault;
import dk.sundhedsdatastyrelsen.dcc.wsdl.DccNTSServicePort;
import dk.sundhedsdatastyrelsen.dcc.xsd.DccRequestType;
import org.w3c.dom.Node;

import javax.annotation.PreDestroy;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import java.net.URISyntaxException;
import java.util.Optional;

public class DccNTSServicePortFactory {

    public static DccNTSServicePort buildDccNTSServicePort(String dccHost, String soapAction, Optional<String> serviceIdentifier, Identity identity, String addressingTo) {

        String endpoint = dccHost + "/decoupling" + serviceIdentifier.map(si -> "/service/" + si + "/").orElse("/");
        try {
            return new DccNTSServiceTestClient(endpoint, soapAction, addressingTo, identity);
        } catch (JAXBException |URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    public static DccNTSServicePort buildDccNTSServicePort(String dccHost, String soapAction, Optional<String> serviceIdentifier) {

        return buildDccNTSServicePort(dccHost, soapAction, serviceIdentifier, EmployeeIdentities.lægeMargaretHamilton(),null);
    }

   public static class DccNTSServiceTestClient implements DccNTSServicePort, AutoCloseable {

        private final WsTestClient nspSoapClient;

        private final String endpoint;

        private final String soapAction;

        public DccNTSServiceTestClient(String endpoint, String soapAction, String addressingTo, Identity identity) throws URISyntaxException, JAXBException {
            this.nspSoapClient = new WsTestClient(JAXBContext.newInstance(DccRequestType.class), addressingTo, identity);
            this.endpoint = endpoint;
            this.soapAction = soapAction;
        }

       @Override
       public String dccCall(DccRequestType dccRequest) throws DGWSFault {
           try {
               Node body = nspSoapClient.callService(endpoint, DccRequestType.class, "http://www.sundhedsdatastyrelsen.dk/dcc/xsd", "dccRequest", dccRequest, soapAction);
               Node ok = body.getOwnerDocument().getElementsByTagName("OK").item(0);
               return ok.getTextContent();
           } catch (WebServiceException e) {
               throw new DGWSFault(e.getFaultCode(), e.getMessage());
           }
       }

       @Override
       @PreDestroy
       public void close() throws Exception {
           nspSoapClient.close();
       }
   }
}
