package dk.rsd.dcc.util;

import dk.nsp.test.idp.NspSoapClient;
import dk.nsp.test.idp.model.Identity;
import dk.sosi.seal.xml.XmlUtil;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.namespace.QName;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.function.Consumer;

public class WsTestClient implements AutoCloseable {
    private final NspSoapClient nspSoapClient;

    private final JAXBContext jaxbContext;

    private final String addressingTo;

    private final Identity identity;

    public WsTestClient(JAXBContext jaxbContext, String addressingTo, Identity identity) {
        Consumer<String> logger = message -> Logger.getLogger(WsTestClient.class).info(message);
        this.jaxbContext = jaxbContext;
        this.nspSoapClient = new NspSoapClient(logger);
        this.addressingTo = addressingTo;
        this.identity = identity;
    }

    public <RequestType> Node callService(String endpoint, Class<RequestType> requestClazz, String requestNamespace, String requestLocalPart, RequestType request, String soapAction) throws dk.rsd.dcc.util.WebServiceException {

        NspSoapClient.NspSoapService service = null;
        try {
            service = nspSoapClient.request(new URI(endpoint), soapAction);
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }   
        Document requestDom = dk.sosi.seal.xml.XmlUtil.createEmptyDocument();

        List<Element> headers = new LinkedList<>();
        if (addressingTo != null) {
            Element toAddressing = requestDom.createElementNS("http://www.w3.org/2005/08/addressing", "To");
            toAddressing.setTextContent(addressingTo);
            headers.add(toAddressing);

            Element actionAddressing = requestDom.createElementNS("http://www.w3.org/2005/08/addressing", "Action");
            actionAddressing.setTextContent(soapAction);
            headers.add(actionAddressing);
        }
        try {
            JAXBElement<RequestType> jaxbElement = new JAXBElement<RequestType>(new QName(requestNamespace, requestLocalPart), requestClazz, request);
            jaxbContext.createMarshaller().marshal(jaxbElement, requestDom);
        } catch (JAXBException e) { throw new RuntimeException(e); }
        try (NspSoapClient.NspSoapResponse response = (identity != null
                ? service.as(identity).execute(requestDom.getDocumentElement(), headers.toArray(new Element[headers.size()]))
                : service.withoutSecurity().execute(requestDom.getDocumentElement(),  headers.toArray(new Element[headers.size()])))) {
            Document dccResponse = XmlUtil.readXml(new Properties(), new InputSource(response.getResponse()), false);
            Element documentElement = dccResponse.getDocumentElement();
            if (response.isFault()) {
                Node faultCode = documentElement.getElementsByTagNameNS("http://www.medcom.dk/dgws/2006/04/dgws-1.0.xsd", "FaultCode").item(0);
                Node faultString = documentElement.getElementsByTagName("faultstring").item(0);
                throw new WebServiceException(faultString != null ? faultString.getTextContent() : null, faultCode != null ? faultCode.getTextContent() : null);
            }
            return dccResponse.getElementsByTagNameNS("http://schemas.xmlsoap.org/soap/envelope/", "Body").item(0);
        }
        catch (WebServiceException f) { throw f; }
        catch (Exception e) { throw new RuntimeException(e); }
    }

    @Override
    public void close() throws Exception {
        nspSoapClient.close();
    }
}
