/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/client-dcc-websphere/src/test/java/dk/rsd/dcc/client/AsynchTestClient.java $
 * $Id: AsynchTestClient.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.client;

import java.io.StringReader;
import java.rmi.RemoteException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Properties;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import junit.framework.TestCase;
import junit.textui.TestRunner;

import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axiom.soap.impl.builder.StAXSOAPModelBuilder;
import org.apache.axis2.client.Options;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.PatternLayout;
import org.w3._2000._09.xmldsig_.KeyInfo;
import org.w3._2000._09.xmldsig_.X509Data;

import com.ibm.mq.jms.JMSC;
import com.ibm.mq.jms.MQQueue;
import com.ibm.mq.jms.MQQueueConnectionFactory;

import dk.dkma.medicinecard.xml_schema._2009._01._01.MedicineCardOverviewStructureType;
import dk.dkma.medicinecard.xml_schema._2009._01._01.MedicineCardRequestStructureType;
import dk.dkma.medicinecard.xml_schema._2009._01._01.MedicineCardResponseType;
import dk.rsd.dcc.demo.SOSIHandler;
import dk.rsd.dcc.demo.SosiUserData;
import dk.sosi._2007._07.decoupling_component_1_0.DecouplingHeader;
import dk.sosi.gw._2007_09.LogoutRequestBody;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningRequestBody;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.ReplyHeader;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.vault.EmptyCredentialVault;
import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.www.dcc.medicinecard._2009._01._01.DGWSFault;
import dk.sosi.www.dcc.medicinecard._2009._01._01.DccMedicineCardStub;

public class AsynchTestClient extends TestCase {

	//private static final String ENDPOINT = "http://localhost:8080/decoupling/";
	//private static final String ENDPOINT = "http://10.0.1.5:9080/decoupling/";
	//private static final String ENDPOINT = "http://192.168.114.128:9081/decoupling/";
	private static final String ENDPOINT = "http://10.161.2.82:9080/decoupling/"; //Test
	//private static final String ENDPOINT = "http://10.161.2.80:9080/decoupling/"; //QA
	//private static final String ENDPOINT = "http://10.161.2.81:9080/decoupling/"; //Prod

	private static final boolean USE_SOSI = true;

	// Adjust to match your MQ installation - furthermore, you might need to specify a username & password,
	// depending on your MQ configuration
//	private String mqHostname="gasser";
//	private String queueManager="QM_gasser";
//	private String channel = "S_gasser";
//	private String queuename = "DccHarmoniReplyQueue";

	private String mqHostname="10.161.2.76"; //Test
	private String queueManager="RSD001D";

//	private String mqHostname="10.161.2.67"; //QA
//	private String queueManager="RSD001T";

//	private String mqHostname="10.161.2.58"; //Prod
//	private String queueManager="RSD001";
//
	private String channel = "CLIENTS.DCC.HARMONI";
	private String queuename = "RSD005B.DCC.HARMONI.REPLY";

	//private static final String CPR = "1111111118";
	private static final String CPR = "0812926632";

	private DccMedicineCardStub facade;

	protected void setUp() throws Exception {
		super.setUp();
		System.out.println("Starting test ...");

		PatternLayout patternLayout = new PatternLayout("%-5p %c - %m%n");
		ConsoleAppender consoleAppender = new ConsoleAppender(patternLayout);
		consoleAppender.setThreshold(Level.ERROR);
		BasicConfigurator.configure(consoleAppender);

		facade = new DccMedicineCardStub(ENDPOINT);
		if (USE_SOSI) {
			facade._getServiceClient().engageModule("sosi");
			SOSIHandler.setUserData(new SosiUserData());
		}
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		if (USE_SOSI)
			facade._getServiceClient().disengageModule("sosi");
		facade = null;
	}

	private void callMedicineCard() throws Exception {

		Options options = facade._getServiceClient().getOptions();
		options.setProperty(HTTPConstants.SO_TIMEOUT, new Integer(60 * 1000));
		//options.setProperty(org.apache.axis2.transport.http.HTTPConstants.MC_GZIP_REQUEST, Boolean.TRUE);
		options.setProperty(org.apache.axis2.transport.http.HTTPConstants.MC_ACCEPT_GZIP, Boolean.TRUE);
		//options.setProperty(Constants.Configuration.MESSAGE_TYPE, "application/fastinfoset");


		MedicineCardRequestStructureType requestStructureType = new MedicineCardRequestStructureType();
		requestStructureType.setPersonCivilRegistrationIdentifier(CPR);

		DecouplingHeader decouplingHeader = new DecouplingHeader();
		decouplingHeader.setTimeout(5);

		MedicineCardResponseType medicineCard = facade.GetMedicineCard(requestStructureType, null, null, null, decouplingHeader);
		System.out.println("Received synchronous response ...");
	}

	private void logoutFromSosiGW() throws RemoteException {
		LogoutRequestBody logoutRequestBody = new LogoutRequestBody();
		logoutRequestBody.setValue("whatever");
		facade.logout(logoutRequestBody, null, null);
	}

	public void testAsynchCall() throws Exception {
		checkMQconnection();
		if (USE_SOSI)
			loginIntoSosiGW();
		try {
			callMedicineCard();
			fail("'dcc_queued' expected");
		}
		catch (DGWSFault fault){
//			assertEquals("dcc_queued", fault.getFaultMessage());
			assertEquals("dcc_queued_after_timeout", fault.getFaultMessage());
			String response = retrieveResponseFromQueue();
			unmarshallResponse(response);
		} finally {
			if (USE_SOSI)
				logoutFromSosiGW();
		}
	}

	private String retrieveResponseFromQueue() throws JMSException {
		QueueReceiver receiver = null;
		QueueSession queueSession = null;
		QueueConnection queueConnection = null;
		try {
			MQQueueConnectionFactory factory = getMQConnectionFactory();
			queueConnection = factory.createQueueConnection(" ", " ");
			queueConnection.start();
			queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			MQQueue queue = (MQQueue) queueSession.createQueue(queuename);
			receiver = queueSession.createReceiver(queue);
			Message msg = receiver.receive(120 * 1000);
			TextMessage textMsg = (TextMessage) msg;
			String xml = textMsg.getText();
			System.out.println("Received response via MQ ...");
			System.out.println("Message length: " + xml.length());
			System.out.println(xml.substring(0, Math.min(xml.length(), 10000)));
			return xml;
		} finally {
			if (receiver != null) receiver.close();
			if (queueSession != null) queueSession.close();
			if (queueConnection != null) queueConnection.close();
		}
	}

	private void unmarshallResponse(String xml) throws JAXBException, XMLStreamException, FactoryConfigurationError {
		SOSIFactory sosiFactory = new SOSIFactory(new EmptyCredentialVault(), new Properties());
		XMLStreamReader parser = XMLInputFactory.newInstance().createXMLStreamReader(new StringReader(xml));
		StAXSOAPModelBuilder builder = new StAXSOAPModelBuilder(parser);

		SOAPBody soapBody = builder.getSOAPEnvelope().getBody();
		if (soapBody.hasFault()) {
			System.out.println("Fault: " + soapBody.getFault()) ;
			// handle fault ....
		} else {
			SOAPHeader soapHeader = builder.getSOAPEnvelope().getHeader();
			ReplyHeader replyHeader = sosiFactory.deserializeReplyHeader(soapHeader.toString());
			String requestID = replyHeader.getRequestID();
			assertEquals(SOSIHandler.getLastMessageID(), requestID);
			String responseType = soapBody.getFirstElement().getLocalName();
			System.out.println("ResponseType: " + responseType);
			if (responseType.equals("MedicineCardResponseStructure")){
				JAXBContext context = JAXBContext.newInstance(MedicineCardResponseType.class);
				Unmarshaller unmarshaller = context.createUnmarshaller();
				XMLStreamReader streamReader = soapBody.getFirstElement().getXMLStreamReaderWithoutCaching();
				MedicineCardResponseType result = (MedicineCardResponseType) unmarshaller.unmarshal(streamReader, MedicineCardResponseType.class).getValue();
				List<MedicineCardOverviewStructureType> medicineCardOverviewStructure = result.getMedicineCardOverviewStructure();
				MedicineCardOverviewStructureType structureType = medicineCardOverviewStructure.get(0);
				long versionIdentifier = structureType.getMedicineCardVersionIdentifier();
				System.out.println("MedicinCardVersion: " + versionIdentifier);
			} else {
				// handle other responses .....
			}
		}
	}

	private void checkMQconnection() throws JMSException {
		QueueSender sender = null;
		QueueSession queueSession = null;
		QueueConnection queueConnection = null;
		try {
			MQQueueConnectionFactory factory = getMQConnectionFactory();
			queueConnection = factory.createQueueConnection();
			queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			MQQueue queue = (MQQueue) queueSession.createQueue(queuename);
			queue.setTargetClient(JMSC.MQJMS_CLIENT_NONJMS_MQ);
			sender = queueSession.createSender(queue);
		} finally {
			if (sender != null) sender.close();
			if (queueSession != null) queueSession.close();
			if (queueConnection != null) queueConnection.close();
		}
	}

	private MQQueueConnectionFactory getMQConnectionFactory() throws JMSException {
		MQQueueConnectionFactory factory = new MQQueueConnectionFactory();
		factory.setHostName(mqHostname);
		factory.setQueueManager(queueManager);
		factory.setChannel(channel);
		factory.setTransportType(JMSC.MQJMS_TP_CLIENT_MQ_TCPIP);
		return factory;
	}

	private void loginIntoSosiGW() throws RemoteException, DGWSFault, NoSuchAlgorithmException, NoSuchProviderException, InvalidKeyException,
			SignatureException, CertificateEncodingException {
		RequestIdCardDigestForSigningRequestBody body = new RequestIdCardDigestForSigningRequestBody();
		body.setValue("whatever");

		RequestIdCardDigestForSigningResponse signingResponse = facade.requestIdCardDigestForSigning(body, null, null);
		byte[] digestValue = signingResponse.getDigestValue();

		// As a convenience GenericCredentialVault is used to load the
		// certificate
		Properties properties = SignatureUtil.setupCryptoProviderForJVM();
		GenericCredentialVault vault = new GenericCredentialVault(properties, "!234Qwer");
		vault.setSystemCredentialPair(this.getClass().getResourceAsStream("/cvr-19343634.pfx"), "!234Qwer");

		// In a application setup, the signing would be involve user-interaction
		Signature jceSign = Signature.getInstance("SHA1withRSA", SignatureUtil.getCryptoProvider(properties,
				SOSIFactory.PROPERTYNAME_SOSI_CRYPTOPROVIDER_SHA1WITHRSA));
		PrivateKey key = vault.getSystemCredentialPair().getPrivateKey();
		jceSign.initSign(key);
		jceSign.update(digestValue);

		byte[] signedBytes = jceSign.sign();
		X509Certificate certificate = vault.getSystemCredentialPair().getCertificate();

		X509Data data = new X509Data();
		data.setX509Certificate(certificate.getEncoded());

		KeyInfo keyInfo = new KeyInfo();
		keyInfo.setX509Data(data);

		SignIdCardRequestBody signIdCardRequestBody = new SignIdCardRequestBody();
		signIdCardRequestBody.setKeyInfo(keyInfo);
		signIdCardRequestBody.setSignatureValue(signedBytes);

		facade.signIdCard(signIdCardRequestBody, null, null);
	}

	public static void main(String[] args) throws Exception {
		TestRunner.run(AsynchTestClient.class);
	}

}
