/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/client-dcc-websphere/src/test/java/dk/rsd/dcc/client/WebsphereMQClient.java $
 * $Id: WebsphereMQClient.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.client;

import java.util.Enumeration;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.QueueBrowser;
import javax.jms.QueueConnection;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;

import com.ibm.mq.jms.JMSC;
import com.ibm.mq.jms.MQQueue;
import com.ibm.mq.jms.MQQueueConnectionFactory;

public class WebsphereMQClient {

	public static final int MQ_DEFAULT_PORT = 1414;

	private MQQueueConnectionFactory factory;

	public WebsphereMQClient(String hostname, String queueManager, String channel, int port) throws JMSException {
		factory = new MQQueueConnectionFactory();
		factory.setHostName(hostname);
		factory.setQueueManager(queueManager);
		factory.setPort(port);
		factory.setChannel(channel);
		factory.setTransportType(JMSC.MQJMS_TP_CLIENT_MQ_TCPIP);
	}

	public WebsphereMQClient(String hostname, String queueManager, String channel) throws JMSException {
		this(hostname, queueManager, channel, MQ_DEFAULT_PORT);
	}

	public void send(String queueName, String text, String username, String password) throws JMSException {
		QueueConnection queueConnection = null;
		QueueSession queueSession = null;
		QueueSender sender = null;
		try {
			if (username != null && password != null) {
				queueConnection = factory.createQueueConnection(username, password);
			} else {
				queueConnection = factory.createQueueConnection();
			}
			queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			MQQueue queue = (MQQueue) queueSession.createQueue(queueName);
			queue.setTargetClient(JMSC.MQJMS_CLIENT_NONJMS_MQ);
			sender = queueSession.createSender(queue);
			TextMessage message = queueSession.createTextMessage();
			message.setText(text);
			sender.send(message);
		} finally {
			try { if (sender != null) sender.close(); } catch (JMSException e) {/* ignore */};
			try { if (queueSession != null) queueSession.close(); } catch (JMSException e) {/* ignore */};
			try { if (queueConnection != null) queueConnection.close(); } catch (JMSException e) {/* ignore */};
		}
	}

	private String receive(String queueName) throws JMSException {
		QueueReceiver receiver = null;
		QueueSession queueSession = null;
		QueueConnection queueConnection = null;
		try {
			queueConnection = factory.createQueueConnection();
			queueConnection.start();
			queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			MQQueue queue = (MQQueue) queueSession.createQueue(queueName);
			receiver = queueSession.createReceiver(queue);
			Message msg = receiver.receive(10 * 1000);
			TextMessage textMsg = (TextMessage) msg;
			String xml = textMsg.getText();
			System.out.println(xml.substring(0, 1200));
			return xml;
		} finally {
			try { if (receiver != null) receiver.close(); } catch (JMSException e) {/* ignore */};
			try { if (queueSession != null) queueSession.close(); } catch (JMSException e) {/* ignore */};
			try { if (queueConnection != null) queueConnection.close(); } catch (JMSException e) {/* ignore */};
		}
	}

	private boolean browse(String queueName) throws JMSException {
		QueueBrowser browser = null;
		QueueSession queueSession = null;
		QueueConnection queueConnection = null;
		try {
			queueConnection = factory.createQueueConnection();
			queueConnection.start();
			queueSession = queueConnection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			MQQueue queue = (MQQueue) queueSession.createQueue(queueName);
			browser = queueSession.createBrowser(queue);
			Enumeration messages = browser.getEnumeration();
			System.out.println(messages.hasMoreElements());
			return messages.hasMoreElements();
		} finally {
			try { if (browser != null) browser.close(); } catch (JMSException e) {/* ignore */};
			try { if (queueSession != null) queueSession.close(); } catch (JMSException e) {/* ignore */};
			try { if (queueConnection != null) queueConnection.close(); } catch (JMSException e) {/* ignore */};
		}
	}

	public static void main(String[] args) throws JMSException {
//		WebsphereMQClient client = new WebsphereMQClient("gasser", "QM_gasser", "S_gasser");
//		client.send("DccHarmoniReplyQueue", "foo", null, null);

//		WebsphereMQClient client = new WebsphereMQClient("10.161.2.76", "RSD001D", "CLIENTS.DCC.HARMONI"); //Test
		WebsphereMQClient client = new WebsphereMQClient("10.161.2.67", "RSD001T", "CLIENTS.DCC.HARMONI"); //QA
//		WebsphereMQClient client = new WebsphereMQClient("10.161.2.58", "RSD001", "CLIENTS.DCC.HARMONI"); //Prod
		//client.send("RSD005B.DCC.HARMONI.REPLY", "foo", null, null);
		client.browse("RSD005B.DCC.HARMONI.REPLY");
//		while (client.browse("RSD005B.DCC.HARMONI.REPLY"))
//			client.receive("RSD005B.DCC.HARMONI.REPLY");
	}

}
