#!/bin/sh

# Read configuration file if specified as parameter
if [[ "x$1" != "x" && -r $1 ]]; then
    echo Reading configuration from $1
    . $1
fi

# Set default values
if [ "x$DCC_JBOSS_HOME" = "x" ]; then
    DCC_JBOSS_HOME=/Users/chg/products/jboss-4.2.3.GA
fi

if [ "x$DCC_JBOSS_PORT" = "x" ]; then
    DCC_JBOSS_PORT=8080
fi

if [ "x$DCC_JBOSS_NAMING_PORT" = "x" ]; then
    DCC_JBOSS_NAMING_PORT=1099
fi

if [ "x$DCC_CLOVER_HISTORY_DIR" = "x" ]; then
    DCC_CLOVER_HISTORY_DIR=clover_history
fi

echo ----------------------------------------------------
echo DCC_JBOSS_HOME: $DCC_JBOSS_HOME
echo DCC_JBOSS_PORT: $DCC_JBOSS_PORT
echo DCC_JBOSS_NAMING_PORT: $DCC_JBOSS_NAMING_PORT
echo DCC_CLOVER_HISTORY_DIR: $DCC_CLOVER_HISTORY_DIR
echo ----------------------------------------------------

DCC_ENDPOINT=http://localhost:$DCC_JBOSS_PORT/decoupling-clover/

# Ensure that there is no sosi-dcc instance deployed
rm -f $DCC_JBOSS_HOME/server/default/deploy/decoupling*.war

# Start JBoss
$DCC_JBOSS_HOME/bin/run.sh  2>&1 > jboss.log &

# Build an instrumented sosi-dcc instance
mvn clean clover2:instrument

# Deploy the instrumented sosi-dcc instance
cp target/clover/decoupling-clover.war $DCC_JBOSS_HOME/server/default/deploy/

# Measure test coverage
mvn clover2:setup test clover2:aggregate clover2:clover -P coverage -Ddcc.endpoint=$DCC_ENDPOINT -Ddcc.jboss_naming_port=$DCC_JBOSS_NAMING_PORT

# Undeploy the instrumented sosi-dcc instance
rm -f $DCC_JBOSS_HOME/server/default/deploy/decoupling-clover.war

# Stop JBoss
$DCC_JBOSS_HOME/bin/shutdown.sh -s jnp://localhost:$DCC_JBOSS_NAMING_PORT

# Verify that the test coverage target is reached
mvn clover2:check clover2:save-history -Dclover.history.dir=$DCC_CLOVER_HISTORY_DIR
