/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/main/java/dk/rsd/dcc/DccContext.java $
 * $Id: DccContext.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc;

import dk.rsd.dcc.routing.WSAHeaderProcessing;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.util.AbstractMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class DccContext implements Serializable {

	private static final long serialVersionUID = -3448534619162120410L;

	private DecouplingInfo decouplingInfo;
	private String itSystemName;
	private String decoupledRequest;
	private String messageID = "unknown";
	private String flowID = "unknown";
	private String soapAction;
    private String serviceIdentifier;
	private URL serviceURL;
	private URL proxyURL;
	private String dgwsVersion = DccConfig.getInstance().getDGWSVersion();
    private String clientIP = "unknown";
    private String contentType;
    private byte[] decoupledBytes;
    // List of <header, value> pairs - in order to support duplicate http headers
    private List<Map.Entry<String, String>> preservedHttpHeaders = new LinkedList<Map.Entry<String, String>>();
    private boolean doNotVerifySSLHostname;
    private WSAHeaderProcessing wsaHeaderProcessing;

    private transient String responseContentType;
    private transient int responseContentLength;
    private transient int responseStatusCode;
    private transient String responseContentEncoding;

    public String getItSystemName() {
		return itSystemName;
	}

	public void setItSystemName(String itSystemName) {
		this.itSystemName = itSystemName;
	}

	public DecouplingInfo getDecouplingInfo() {
		return decouplingInfo;
	}

	public void setDecoupledRequest(String decoupledRequest) {
		this.decoupledRequest = decoupledRequest;
	}

	public void setTimeout(int timeout) {
		internalGetDecouplingInfo().setTimeout(timeout);
	}

	private DecouplingInfo internalGetDecouplingInfo() {
		if (decouplingInfo == null) {
			decouplingInfo = new DecouplingInfo();
		}
		return decouplingInfo;
	}

	public void setMessageID(String messageID) {
		this.messageID = messageID;
	}

	public String getMessageID() {
		return messageID;
	}

	public String getFlowID() {
		return flowID;
	}

	public void setFlowID(String flowID) {
		this.flowID = flowID;
	}

	public String getSoapAction() {
		return soapAction;
	}

	public void setSoapAction(String soapAction) {
		this.soapAction = soapAction;
	}

    public String getServiceIdentifier() {
        return serviceIdentifier;
    }

    public void setServiceIdentifier(String serviceIdentifier) {
        this.serviceIdentifier = serviceIdentifier;
    }

    public URL getServiceURL() {
		return serviceURL;
	}

	public void setServiceURL(URL serviceURL) {
		this.serviceURL = serviceURL;
	}

	public URL getProxyURL() {
		return proxyURL == null ? serviceURL : proxyURL;
	}

	public void setProxyURL(URL proxyURL) {
		this.proxyURL = proxyURL;
	}

	public void setResponseContentType(String contentType) {
		this.responseContentType = contentType;
	}

	public void setResponseContentLength(int contentLength) {
		this.responseContentLength = contentLength;
	}

	public void setResponseStatusCode(int statusCode) {
		this.responseStatusCode = statusCode;
	}

	public String getResponseContentType() {
		return responseContentType;
	}

	public int getResponseContentLength() {
		return responseContentLength;
	}

	public int getResponseStatusCode() {
		return responseStatusCode;
	}

	public void setDGWSVersion(String dgwsVersion) {
		this.dgwsVersion = dgwsVersion;
	}

	public String getDgwsVersion() {
		return dgwsVersion;
	}

	public void setResponseContentEncoding(String contentEncoding) {
		this.responseContentEncoding = contentEncoding;
	}

	public String getResponseContentEncoding() {
		return responseContentEncoding;
	}

	public boolean isResponseGzipEncoded() {
		return responseContentEncoding != null && responseContentEncoding.equalsIgnoreCase("gzip");
	}

    public String getClientIP() {
        return clientIP;
    }

    public void setClientIP(String clientIP) {
        this.clientIP = clientIP;
    }

    public void setContentType(String contentType) {
        this.contentType = contentType;
    }

    public String getContentType() {
        return contentType;
    }

    public void setDecoupledRequestAsBytes(byte[] decoupledBytes) {
        this.decoupledBytes = decoupledBytes;
    }

    public byte[] getBytesToSend() throws UnsupportedEncodingException {
        return decoupledBytes != null ? decoupledBytes : decoupledRequest.getBytes("UTF-8");
    }

    public String getDecoupledRequestAsString() throws UnsupportedEncodingException {
        return decoupledRequest != null ? decoupledRequest : new String(decoupledBytes, "UTF-8");
    }

    public void addPreservedHttpHeader(String header, String value) {
        preservedHttpHeaders.add(new AbstractMap.SimpleEntry<String, String>(header, value));
    }

    public List<Map.Entry<String, String>> getPreservedHttpHeaders() {
        return preservedHttpHeaders;
    }

    public void setDoNotVerifySSLHostname(boolean doNotVerifySSLHostname) {
        this.doNotVerifySSLHostname = doNotVerifySSLHostname;
    }

    public boolean doNotVerifySSLHostname() {
        return doNotVerifySSLHostname;
    }

    public void setWSAHeaderProcessing(WSAHeaderProcessing wsaHeaderProcessing) {
        this.wsaHeaderProcessing = wsaHeaderProcessing;
    }

    public WSAHeaderProcessing getWSAHeaderProcessing() {
        return wsaHeaderProcessing;
    }
}
