/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/main/java/dk/rsd/dcc/DccUtil.java $
 * $Id: DccUtil.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc;

import dk.rsd.dcc.fault.DccFault;
import dk.rsd.dcc.fault.GeneralFault;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.Reply;
import dk.sosi.seal.vault.EmptyCredentialVault;
import dk.sosi.seal.xml.XmlUtil;
import org.apache.log4j.Logger;

import java.io.*;
import java.util.zip.GZIPInputStream;

public class DccUtil {

	//TODO Check IOException handling

	private static final Logger log = Logger.getLogger(DccUtil.class);

	private static SOSIFactory factory = new SOSIFactory(new EmptyCredentialVault(), System.getProperties());

	private DccUtil() {
	}

	public static String createErrorReply(DccContext context, DccFault fault) {
		return createErrorReply(context, fault.getFaultCode(), fault.getMessage());
	}

	public static String createErrorReply(DccContext context, String faultCode, String msg) {
		Reply reply = factory.createNewErrorReply(context.getDgwsVersion(), context.getMessageID(), context.getFlowID(), faultCode, msg);
		return XmlUtil.node2String(reply.serialize2DOMDocument());
	}

	public static long streamResponseFromService(InputStream inputStream, OutputStream outputStream, boolean doGunzip, boolean isGzipped) throws DccFault, IOException  {
		long bytesCopied = 0;
		if (doGunzip) {
			inputStream = new GZIPInputStream(inputStream);
		}
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			byte[] buffer = new byte[8192];
			int n;
			while ((n = inputStream.read(buffer, 0, buffer.length)) != -1) {
				outputStream.write(buffer, 0, n);
				if (log.isDebugEnabled()) {
					baos.write(buffer, 0, n);
				}
				bytesCopied += n;
			}
		} catch (IOException e) {
		    log.error("streamResponseFromService got an error. (doGunzip:"+doGunzip+";isGzipped:"+isGzipped+")", e);
			throw e;
		} finally {
			try { inputStream.close();} catch (IOException e) {/* ignore */}
		}
		if (log.isDebugEnabled()) {
			try {
				String responseStr;
				if (baos.size() > 0 && isGzipped && ! doGunzip) {
					responseStr = consumeStreamToString(new GZIPInputStream(new ByteArrayInputStream(baos.toByteArray())));
				} else {
					responseStr = baos.toString("UTF-8");
				}
				log.debug("Response from service: " + (responseStr.length() == 0 ? "<empty response>" : responseStr));
//				try {
//					log.debug("Checking DGWS compliance ....");
//					SOSIFactory factory = new SOSIFactory(new EmptyCredentialVault(), SignatureUtil.setupCryptoProviderForJVM());
//					factory.deserializeReply(responseStr);
//					log.debug("Response is DGWS compliant");
//				} catch (ModelException e) {
//					log.debug("Response is NOT compliant", e);
//				} catch (XmlUtilException e) {
//					log.debug("Response is NOT compliant", e);
//				}
			} catch (UnsupportedEncodingException e) {
				log.error("Unsupported encoding..", e);
				throw new GeneralFault(e);
			}
		}
		return bytesCopied;
	}

	public static String consumeStreamToString(InputStream is) throws IOException {
        ByteArrayOutputStream baos = consumeToByteArrayOutputStream(is);
		return baos.toString("UTF-8");
	}

    public static byte[] consumeStreamToByteArray(InputStream is) throws IOException {
        ByteArrayOutputStream baos = consumeToByteArrayOutputStream(is);
        return baos.toByteArray();
    }

    private static ByteArrayOutputStream consumeToByteArrayOutputStream(InputStream is) throws IOException {
        is = new BufferedInputStream(is);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        byte[] buffer = new byte[8192];
        int n;
        while ((n = is.read(buffer, 0, buffer.length)) != -1) {
            baos.write(buffer, 0, n);
        }
        return baos;
    }

}
