/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/main/java/dk/rsd/dcc/dispatch/AbstractDispatcher.java $
 * $Id: AbstractDispatcher.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.dispatch;

import commonj.work.WorkException;
import commonj.work.WorkItem;
import commonj.work.WorkManager;
import dk.rsd.dcc.DccContext;
import dk.rsd.dcc.fault.DccFault;
import dk.rsd.dcc.fault.GeneralFault;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.InputStream;

import static dk.rsd.dcc.fault.FaultUtil.formatDestination;

public abstract class AbstractDispatcher implements Dispatcher{

	private static final Logger log = Logger.getLogger(AbstractDispatcher.class);

	private WorkManager workManager;
	private WorkItem workItem;
	private DccWork dccWork;

	public InputStream dispatch(DccContext context) throws DccFault {
		initialize(context);

		try {
			workManager = CommonResourceLocator.getWorkManager();
			log.debug("Starting work ...");
			dccWork = new DccServiceCall(context);
			workItem = workManager.schedule(dccWork);

			InputStream inputStream = execute(context);
			return inputStream;

		} catch (WorkException e) {
			log.error("Error calling service " + formatDestination(context, false), e);
			throw new GeneralFault("Error calling service " + formatDestination(context, true), e);
		} catch (IOException e) {
            log.error("Error calling service " + formatDestination(context, false), e);
            throw new GeneralFault("Error calling service " + formatDestination(context, true), e);
		} catch (InterruptedException e) {
            log.error("Error calling service " + formatDestination(context, false), e);
            throw new GeneralFault("Error calling service " + formatDestination(context, true), e);
		}


	}

	protected abstract void initialize(DccContext context) throws DccFault;

	protected abstract InputStream execute(DccContext context) throws DccFault, WorkException, IOException, InterruptedException;

	protected WorkManager getWorkManager() {
		return workManager;
	}

	protected WorkItem getWorkItem() {
		return workItem;
	}

	protected DccWork getDccWork() {
		return dccWork;
	}

}
