/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/main/java/dk/rsd/dcc/dispatch/SynchronousDispatcher.java $
 * $Id: SynchronousDispatcher.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.dispatch;

import commonj.work.WorkEvent;
import commonj.work.WorkException;
import dk.rsd.dcc.DccContext;
import dk.rsd.dcc.dispatch.DccWork.ProxyIOException;
import dk.rsd.dcc.fault.DccFault;
import dk.rsd.dcc.fault.GeneralFault;
import dk.rsd.dcc.fault.TimeoutFault;
import org.apache.log4j.Logger;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;

import static dk.rsd.dcc.fault.FaultUtil.formatDestination;

public class SynchronousDispatcher extends AbstractDispatcher {

	private static final Logger log = Logger.getLogger(SynchronousDispatcher.class);

	protected void initialize(DccContext context) {
		// Do nothing
	}

	protected InputStream execute(DccContext context) throws DccFault, InterruptedException, WorkException {
		int timeout = context.getDecouplingInfo().getTimeout();
		Collection workItems = new ArrayList();
		workItems.add(getWorkItem());
		try {
			getWorkManager().waitForAll(workItems, timeout);
			if (getWorkItem().getStatus() == WorkEvent.WORK_COMPLETED) {
				DccWork dccWork = (DccWork) getWorkItem().getResult();
				return dccWork.getResponseStream();
			} else {
				getDccWork().release();
                log.info("Timeout in DCC reached while calling service " + formatDestination(context, false));
				throw new TimeoutFault("Timeout in DCC reached while calling service " + formatDestination(context, true));
			}
		} catch (ProxyIOException e) {
			log.error("Error in DCC calling service " + formatDestination(context, false), e);
			throw new GeneralFault("Error in DCC calling service " + formatDestination(context, true), e);
		}
	}

}
