/*
 *  The MIT License
 *  Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/main/java/dk/rsd/dcc/dispatch/ssl/SSLDispatcherContext.java $
 * $Id: SSLDispatcherContext.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.dispatch.ssl;

import org.apache.log4j.Logger;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.TrustManagerFactory;
import javax.net.ssl.X509TrustManager;
import java.security.*;

/**
 * Created by chg on 21/04/2016.
 */
public class SSLDispatcherContext {

    private static final String TLS_PROTOCOL = "TLS";

    private static Logger log = Logger.getLogger(SSLDispatcherContext.class);

    private static SSLContext sslContext;

    //For unit-testing purposes
    /*pp*/ static void reset() {
        sslContext = null;
    }

    public static synchronized SSLContext getSSLContext() {
        if (sslContext == null) {
            throw new IllegalStateException("SSLDispatcherContext is not initalized");
        }
        return sslContext;
    }

    public static synchronized void init(KeyStore trustStore) throws GeneralSecurityException {
        if (trustStore == null) {
            log.info("No SSL truststore defined containing additional trusted certificates or CAs. Using only Java's own trusted CAs to check trust for outgoing SSL connections.");
            sslContext = SSLContext.getDefault();
            return;
        }

        X509TrustManager defaultTrustManager = getX509TrustManager(null);
        X509TrustManager additionalTrustManager = getX509TrustManager(trustStore);

        CompositeX509TrustManager compositeTrustManager = new CompositeX509TrustManager(new X509TrustManager[]{defaultTrustManager, additionalTrustManager});

        SSLContext newSSLContext = SSLContext.getInstance(TLS_PROTOCOL);
        newSSLContext.init(null, new TrustManager[]{compositeTrustManager}, null);

        sslContext = newSSLContext;

    }

    private static X509TrustManager getX509TrustManager(KeyStore trustStore) throws NoSuchAlgorithmException, KeyStoreException {
        TrustManagerFactory factory = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
        // If 'trustStore' is null the factory is initialised with the default trust store
        factory.init(trustStore);

        X509TrustManager x509TrustManager = null;
        for (TrustManager trustManager : factory.getTrustManagers()) {
            if (trustManager instanceof X509TrustManager) {
                x509TrustManager = (X509TrustManager) trustManager;
                break;
            }
        }
        return x509TrustManager;
    }
}
