package dk.rsd.dcc.routing;/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2014 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/main/java/dk/rsd/dcc/routing/RoutingConfiguration.java $
 * $Id: RoutingConfiguration.java 33 2017-01-12 16:52:05Z chg $
 */

import dk.rsd.dcc.fault.DccFault;
import dk.rsd.dcc.DecouplingInfo;
import dk.rsd.dcc.fault.GeneralFault;
import org.apache.log4j.Logger;

import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class RoutingConfiguration {

    private static final Logger log = Logger.getLogger(RoutingConfiguration.class);

    private static final Route EMPTY_ROUTE = new Route();

    private Map<RouteIdentifier, Route> routeMap;

    public RoutingConfiguration(List<Route> routes) {
        routeMap = new HashMap<>();
        for (Route route : routes) {
            routeMap.put(RouteIdentifier.fromRoute(route), route);
        }
    }

    public URL getServiceURL(String soapAction, String serviceIdentifier) throws DccFault {
        URL serviceURL = safeGetRoute(soapAction, serviceIdentifier).getServiceURL();
        if (serviceURL == null) {
            String msg = "No service endpoint specified for " + formatParameters(soapAction, serviceIdentifier) + " in decoupling service.";
            log.error(msg);
            throw new GeneralFault(msg);
        }
        return serviceURL;
    }

    public URL getProxyURL(String soapAction, String serviceIdentifier) throws DccFault {
        return safeGetRoute(soapAction, serviceIdentifier).getProxyURL();
    }

    public DecouplingInfo getDefaultDecouplingInfo(String soapAction, String serviceIdentifier) throws DccFault {
        Route route = safeGetRoute(soapAction, serviceIdentifier);

        DecouplingInfo decouplingInfo = route.getDecouplingInfo();
        if (decouplingInfo == null) {
            String msg = "No default decoupling info specified in decoupling service for " + formatParameters(soapAction, serviceIdentifier);
            log.warn(msg);
            throw new GeneralFault(msg);
        }
        return decouplingInfo;

    }

    public int getIdCardMaxAgeInMinutes(String soapAction, String serviceIdentifier) {
        return safeGetRoute(soapAction, serviceIdentifier).getIdCardMaxAge();
    }

    public boolean doNotVerifySSLHostname(String soapAction, String serviceIdentifier) {
        return safeGetRoute(soapAction, serviceIdentifier).doNotVerifySSLHostname();
    }

    public WSAHeaderProcessing getWSAHeaderProcessing(String soapAction, String serviceIdentifier) {
        return safeGetRoute(soapAction, serviceIdentifier).getWSAHeaderProcessing();
    }

    private Route safeGetRoute(String soapAction, String serviceIdentifier) {
        Route route = routeMap.get(new RouteIdentifier(soapAction, serviceIdentifier));
        return route != null ? route : EMPTY_ROUTE;
    }

    private String formatParameters(String soapAction, String serviceIdentifier) {
        return "SOAPAction '" + soapAction + "', ServiceIdentifier '" + (serviceIdentifier != null ? serviceIdentifier : "<empty-serviceidentifier>") + "'";
    }
}
