package dk.rsd.dcc.client;

import dk.dkma.medicinecard.xml_schema._2009._01._01.MedicineCardRequestStructureType;
import dk.dkma.medicinecard.xml_schema._2009._01._01.MedicineCardResponseType;
import dk.dkma.medicinecard.xml_schema._2009._01._01.NewMedicineCardResponseStructureType;
import dk.rsd.dcc.demo.SOSIHandler;
import dk.rsd.dcc.demo.SosiUserData;
import dk.sosi._2007._07.decoupling_component_1_0.DecouplingHeader;
import dk.sosi.gw._2007_09.LogoutRequestBody;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningRequestBody;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.RequestHeader;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.vault.EmptyCredentialVault;
import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.seal.xml.XmlUtil;
import dk.sosi.www.dcc.medicinecard._2009._01._01.DGWSFault;
import dk.sosi.www.dcc.medicinecard._2009._01._01.DccMedicineCardStub;
import junit.framework.TestCase;
import junit.textui.TestRunner;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPFault;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axis2.client.Options;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.log4j.*;
import org.w3._2000._09.xmldsig_.KeyInfo;
import org.w3._2000._09.xmldsig_.X509Data;

import javax.xml.namespace.QName;
import java.rmi.RemoteException;
import java.security.*;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Properties;

public class FMK12GatewayTestClient extends TestCase {

	//private static final String ENDPOINT = "http://localhost:8080/decoupling/";
	private static final String ENDPOINT = "http://192.168.114.134:8080/decoupling/";
	//private static final String ENDPOINT = "http://10.161.2.82:9080/decoupling/"; //Test
	//private static final String ENDPOINT = "http://10.161.2.80:9080/decoupling/"; //QA
	//private static final String ENDPOINT = "http://10.161.2.81:9080/decoupling/"; //Prod
	//private static final String ENDPOINT = "http://10.2.5.245:8080/decoupling/"; //RNJ Test


	//private static final String CPR = "1111111118";
	private static final String CPR = "0812926632";

	private DccMedicineCardStub facade;
	private static Logger log = Logger.getLogger(GatewayTestClient.class);

	protected void setUp() throws Exception {
		super.setUp();

		PatternLayout patternLayout = new PatternLayout("%-5p %c - %m%n");
		ConsoleAppender consoleAppender = new ConsoleAppender(patternLayout);
		consoleAppender.setThreshold(Level.INFO);
		BasicConfigurator.configure(consoleAppender);

		facade = new DccMedicineCardStub(ENDPOINT);
		facade._getServiceClient().engageModule("sosi");
		SOSIHandler.setUserData(new SosiUserData());

		Options options = facade._getServiceClient().getOptions();
		options.setProperty(HTTPConstants.SO_TIMEOUT, new Integer(90 * 1000));
//		options.setProperty(org.apache.axis2.transport.http.HTTPConstants.MC_GZIP_REQUEST, Boolean.TRUE);
//		options.setProperty(org.apache.axis2.transport.http.HTTPConstants.MC_ACCEPT_GZIP, Boolean.TRUE);
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		facade._getServiceClient().disengageModule("sosi");
		facade = null;
	}

	private void callMedicineCard() throws Exception {
		try {
			MedicineCardRequestStructureType requestStructureType = new MedicineCardRequestStructureType();
			requestStructureType.setPersonCivilRegistrationIdentifier(CPR);
			//requestStructureType.setIncludeNonReviewedOnly(true);

			DecouplingHeader decouplingHeader = new DecouplingHeader();
			decouplingHeader.setTimeout(90000);


			MedicineCardResponseType medicineCard = facade.GetMedicineCard(requestStructureType, null, null, null, decouplingHeader);
			System.out.println(medicineCard.getMedicineCardOverviewStructure().get(0).getPatientStructure().getSimpleCPRPerson()
					.getPersonCivilRegistrationIdentifier());
			System.out.println(medicineCard.getMedicineCardOverviewStructure().get(0).getPatientStructure().getAddressPostal()
					.getPostCodeIdentifier());
//			System.out.println(medicineCard.getMedicineCardOverviewStructure().get(0).getDrugMedicationOverviewStructure().get(0).getDrugStructure().getATCStructure().getATCCode());

//			assertEquals(CPR, medicineCard.getMedicineCardOverviewStructure().get(0).getPatientStructure().getSimpleCPRPerson()
//					.getPersonCivilRegistrationIdentifier());

			medicineCard = facade.GetMedicineCard(requestStructureType, null, null, null, null);
//			assertEquals(CPR, medicineCard.getMedicineCardOverviewStructure().get(0).getPatientStructure().getSimpleCPRPerson()
//					.getPersonCivilRegistrationIdentifier());

			NewMedicineCardResponseStructureType newGetMedicineCard = facade.NewGetMedicineCard(requestStructureType, null, null, null, null);
			System.out.println(newGetMedicineCard.getNewMedicineCardOverviewStructure().get(0).getPatientStructure().getSimpleCPRPerson()
					.getPersonCivilRegistrationIdentifier());

			SOSIFactory sosiFactory = new SOSIFactory(new EmptyCredentialVault(), new Properties());
			MessageContext messageContext = facade._getServiceClient().getLastOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_OUT_VALUE);
			SOAPHeader soapHeader = messageContext.getEnvelope().getHeader();
			RequestHeader requestHeader = sosiFactory.deserializeRequestHeader(soapHeader.toString());
			String messageID = requestHeader.getMessageID();
			log.info("MessageID: " + messageID);
//			SOAPHeader returnHeader = facade._getServiceClient().getLastOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE).getEnvelope().getHeader();
//			ReplyHeader replyHeader = sosiFactory.deserializeReplyHeader(returnHeader.toString());
//			log.info(replyHeader.getRequestID());
//			log.info(replyHeader.getMessageID());

		} catch (DGWSFault fault) {
			MessageContext messageContext = facade._getServiceClient().getLastOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
			SOAPFault soapFault = messageContext.getEnvelope().getBody().getFault();
			String faultString = soapFault.getReason().getText();
			log.error("DGWSFault from MedicineCard: " + fault.getFaultMessage() + "/" + faultString);
			throw fault;
		} finally {
			LogoutRequestBody logoutRequestBody = new LogoutRequestBody();
			logoutRequestBody.setValue("whatever");
			facade.logout(logoutRequestBody, null, null);
		}
	}

	public void testImplicitLoginSignInApplication() throws Exception {
		try {
			MedicineCardRequestStructureType requestStructureType = new MedicineCardRequestStructureType();
			requestStructureType.setPersonCivilRegistrationIdentifier(CPR);

			DecouplingHeader decouplingHeader = new DecouplingHeader();
			decouplingHeader.setTimeout(40000);

			facade.GetMedicineCard(requestStructureType, null, null, null, decouplingHeader);
			fail("DGWSFault expected");
		} catch (DGWSFault fault) {
			if (fault.getFaultMessage().equals("sosigw_no_valid_idcard_in_cache")) {
				MessageContext messageContext = facade._getServiceClient().getLastOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
				List headers = (List) messageContext.getProperty(HTTPConstants.HTTP_HEADERS);
				OMElement soapHeader = messageContext.getEnvelope().getHeader();
				OMElement implicitLoginHeader = (OMElement) soapHeader.getFirstChildWithName(new QName("http://sosi.dk/gw/2007.09.01", "implicitLoginHeader"));
				OMElement response = (OMElement) implicitLoginHeader.getFirstChildWithName(new QName("http://sosi.dk/gw/2007.09.01", "requestIdCardDigestForSigningResponse"));
				OMElement digestValue = (OMElement) response.getFirstChildWithName(new QName("http://www.w3.org/2000/09/xmldsig#", "DigestValue"));

				byte[] bytes = XmlUtil.fromBase64(digestValue.getText());
				callSignIDCard(bytes);
				callMedicineCard();

			} else {
				log.error(fault.getFaultMessage());
				throw fault;
			}

		}
	}

	public void testImplicitLoginSignInBrowser() throws Exception {
		try {
			MedicineCardRequestStructureType requestStructureType = new MedicineCardRequestStructureType();
			requestStructureType.setPersonCivilRegistrationIdentifier(CPR);

			DecouplingHeader decouplingHeader = new DecouplingHeader();
			decouplingHeader.setTimeout(20000);

			facade.GetMedicineCard(requestStructureType, null, null, null, decouplingHeader);
			fail("DGWSFault expected");
		} catch (DGWSFault fault) {
			if (fault.getFaultMessage().equals("sosigw_no_valid_idcard_in_cache")) {
				MessageContext messageContext = facade._getServiceClient().getLastOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
				OMElement soapHeader = messageContext.getEnvelope().getHeader();
				OMElement implicitLoginHeader = (OMElement) soapHeader.getFirstChildWithName(new QName("http://sosi.dk/gw/2007.09.01", "implicitLoginHeader"));
				OMElement response = (OMElement) implicitLoginHeader.getFirstChildWithName(new QName("http://sosi.dk/gw/2007.09.01", "requestIdCardDigestForSigningResponse"));
				OMElement browserURL = (OMElement) response.getFirstChildWithName(new QName("http://sosi.dk/gw/2007.09.01", "BrowserUrl"));

				System.out.println("Please open \n" + browserURL.getText() + "\nin your favourite browser, sign the IDcard and then hit any key to continue.");
				System.in.read();

				callMedicineCard();

			} else {
				log.error(fault.getFaultMessage());
				throw fault;
			}

		}
	}

	public void testSignInBrowser() throws Exception {
		RequestIdCardDigestForSigningRequestBody body = new RequestIdCardDigestForSigningRequestBody();
		body.setValue("whatever");

		RequestIdCardDigestForSigningResponse signingResponse = facade.requestIdCardDigestForSigning(body, null, null);
		String browserUrl = signingResponse.getBrowserUrl();
		System.out.println("Please open \n" + browserUrl + "\nin your favourite browser, sign the IDcard and then hit any key to continue.");
		System.in.read();

		callMedicineCard();
	}

	public void testSignInApplication() throws Exception {
		RequestIdCardDigestForSigningRequestBody body = new RequestIdCardDigestForSigningRequestBody();
		body.setValue("whatever");

		try {
			RequestIdCardDigestForSigningResponse signingResponse = facade.requestIdCardDigestForSigning(body, null, null);
			byte[] digestValue = signingResponse.getDigestValue();

			callSignIDCard(digestValue);

			callMedicineCard();
		} catch (DGWSFault fault) {
			MessageContext messageContext = facade._getServiceClient().getLastOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
			SOAPFault soapFault = messageContext.getEnvelope().getBody().getFault();
			String faultString = soapFault.getReason().getText();
			log.error("DGWSFault from MedicineCard: " + fault.getFaultMessage() + " : " + faultString);
			throw fault;
		}
	}

	private void callSignIDCard(byte[] digestValue) throws NoSuchAlgorithmException, NoSuchProviderException,
			InvalidKeyException, SignatureException,
			CertificateEncodingException, RemoteException, DGWSFault {

		// As a convenience GenericCredentialVault is used to laod the
		// certificate
		Properties properties = SignatureUtil.setupCryptoProviderForJVM();
//		GenericCredentialVault vault = new GenericCredentialVault(properties, "!234Qwer");
//		vault.setSystemCredentialPair(this.getClass().getResourceAsStream("/cvr-19343634.pfx"), "!234Qwer");
        GenericCredentialVault vault = new GenericCredentialVault(properties, "Test1234");
        vault.setSystemCredentialPair(this.getClass().getResourceAsStream("/lge1.p12"), "Test1234");

		// In a application setup, the signing would be involve user-interaction
		Signature jceSign = Signature.getInstance("SHA1withRSA", SignatureUtil.getCryptoProvider(properties,
				SOSIFactory.PROPERTYNAME_SOSI_CRYPTOPROVIDER_SHA1WITHRSA));
		PrivateKey key = vault.getSystemCredentialPair().getPrivateKey();
		jceSign.initSign(key);
		jceSign.update(digestValue);

		byte[] signedBytes = jceSign.sign();
		X509Certificate certificate = vault.getSystemCredentialPair().getCertificate();

		X509Data data = new X509Data();
		data.setX509Certificate(certificate.getEncoded());

		KeyInfo keyInfo = new KeyInfo();
		keyInfo.setX509Data(data);

		SignIdCardRequestBody signIdCardRequestBody = new SignIdCardRequestBody();
		signIdCardRequestBody.setKeyInfo(keyInfo);
		signIdCardRequestBody.setSignatureValue(signedBytes);

		facade.signIdCard(signIdCardRequestBody, null, null);

	}

	public static void main(String[] args) throws Exception {
		TestRunner.run(GatewayTestClient.class);
	}

}
