/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/test/java/dk/rsd/dcc/client/IdentityTokenServiceGatewayTestClient.java $
 * $Id: IdentityTokenServiceGatewayTestClient.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.client;

import dk.rsd.dcc.DccUtil;
import dk.rsd.dcc.demo.SOSIHandler;
import dk.rsd.dcc.demo.SosiUserData;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningRequestBody;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.seal.IDWSHFactory;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.*;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;
import dk.sosi.seal.model.dombuilders.IdentityTokenRequestDOMBuilder;
import dk.sosi.seal.pki.SOSITestFederation;
import dk.sosi.seal.vault.EmptyCredentialVault;
import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.seal.xml.XmlUtil;
import dk.sosi.www.dcc.medicinecard._2009._01._01.DGWSFault;
import dk.sosi.www.dcc.medicinecard._2009._01._01.DccMedicineCardStub;
import junit.framework.TestCase;
import org.apache.axiom.soap.SOAPFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.log4j.*;
import org.w3._2000._09.xmldsig_.KeyInfo;
import org.w3._2000._09.xmldsig_.X509Data;

import java.io.IOException;
import java.net.URL;
import java.rmi.RemoteException;
import java.security.*;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.util.Properties;

/**
 * @author $LastChangedBy: chg $ $LastChangedDate: 2017-01-12 17:52:05 +0100 (Thu, 12 Jan 2017) $
 * @version $Revision: 33 $
 */
public class IdentityTokenServiceGatewayTestClient extends TestCase {

	//private static final String ENDPOINT = "http://localhost:8080/decoupling/";
	private static final String ENDPOINT = "http://192.168.114.134:8080/decoupling/";

	// FMK stub used to call SOSI-GW
    private DccMedicineCardStub facade;
    private SosiUserData userData;


	private static Logger log = Logger.getLogger(GatewayTestClient.class);

	protected void setUp() throws Exception {
		super.setUp();

		PatternLayout patternLayout = new PatternLayout("%-5p %c - %m%n");
		ConsoleAppender consoleAppender = new ConsoleAppender(patternLayout);
		consoleAppender.setThreshold(Level.INFO);
		BasicConfigurator.configure(consoleAppender);

		facade = new DccMedicineCardStub(ENDPOINT);
		facade._getServiceClient().engageModule("sosi");
        userData = new SosiUserData();
        SOSIHandler.setUserData(userData);

	}

	protected void tearDown() throws Exception {
		super.tearDown();
		facade._getServiceClient().disengageModule("sosi");
		facade = null;
        userData = null;
	}

	public void testSignInApplication() throws Exception {
		RequestIdCardDigestForSigningRequestBody body = new RequestIdCardDigestForSigningRequestBody();
		body.setValue("whatever");

		try {
			RequestIdCardDigestForSigningResponse signingResponse = facade.requestIdCardDigestForSigning(body, null, null);
			byte[] digestValue = signingResponse.getDigestValue();

			callSignIDCard(digestValue);

			callIdentityTokeService();
		} catch (DGWSFault fault) {
			MessageContext messageContext = facade._getServiceClient().getLastOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
			SOAPFault soapFault = messageContext.getEnvelope().getBody().getFault();
			String faultString = soapFault.getReason().getText();
			log.error("DGWSFault from MedicineCard: " + fault.getFaultMessage() + " : " + faultString);
			throw fault;
		}
	}

    private void callIdentityTokeService() throws IOException {

        URL url = new URL(ENDPOINT);


        CareProvider careProvider = new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, "2343", "dfdsfdsaf");
        UserInfo userInfo = new UserInfo(userData.getCpr(), "Hans", "Dampf", "ererewq", "dfdsfsw", "dfsdafasd", null);
        UserIDCard idcard = new SOSIFactory(new EmptyCredentialVault(), System.getProperties()).createNewUserIDCard("dfdsfdsa", userInfo, careProvider, AuthenticationLevel.MOCES_TRUSTED_USER, null, null, null, null);

        final IDWSHFactory idwshFactory = new IDWSHFactory(null, null);
        IdentityTokenRequestDOMBuilder b = idwshFactory.createIdentityTokenRequestDOMBuilder();
        b.setAudience("http://fmk-online.dk");
        b.setDgwsStyle(true);
        b.setUserIdCard(idcard);
        b.setWSAddressingTo("dddd");
        final String request = XmlUtil.node2String(b.build().getDocumentElement());

        String response = DccUtil.consumeStreamToString(new SoapClient().callService(url, request, "http://sosi.dk/its#Issue"));

        response = response.replace("id=\"Envelope\" ", "");
        System.out.println(response);

        final IdentityTokenResponse identityTokenResponse = idwshFactory.createIdentityTokenResponseModelBuilder().build(XmlUtil.readXml(System.getProperties(), response, false));
        identityTokenResponse.getIdentityToken().validateSignature(new SOSITestFederation(System.getProperties()));

    }

    private void callSignIDCard(byte[] digestValue) throws NoSuchAlgorithmException, NoSuchProviderException,
            InvalidKeyException, SignatureException,
            CertificateEncodingException, RemoteException, DGWSFault {

		// As a convenience GenericCredentialVault is used to laod the
		// certificate
		Properties properties = SignatureUtil.setupCryptoProviderForJVM();
        GenericCredentialVault vault = new GenericCredentialVault(properties, "Test1234");
        vault.setSystemCredentialPair(this.getClass().getResourceAsStream("/lge1.p12"), "Test1234");

		// In a application setup, the signing would be involve user-interaction
		Signature jceSign = Signature.getInstance("SHA1withRSA", SignatureUtil.getCryptoProvider(properties,
				SOSIFactory.PROPERTYNAME_SOSI_CRYPTOPROVIDER_SHA1WITHRSA));
		PrivateKey key = vault.getSystemCredentialPair().getPrivateKey();
		jceSign.initSign(key);
		jceSign.update(digestValue);

		byte[] signedBytes = jceSign.sign();
		X509Certificate certificate = vault.getSystemCredentialPair().getCertificate();

		X509Data data = new X509Data();
		data.setX509Certificate(certificate.getEncoded());

		KeyInfo keyInfo = new KeyInfo();
		keyInfo.setX509Data(data);

		SignIdCardRequestBody signIdCardRequestBody = new SignIdCardRequestBody();
		signIdCardRequestBody.setKeyInfo(keyInfo);
		signIdCardRequestBody.setSignatureValue(signedBytes);

		facade.signIdCard(signIdCardRequestBody, null, null);

	}

}
