/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/test/java/dk/rsd/dcc/client/IntegrationTestSuite.java $
 * $Id: IntegrationTestSuite.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.client;

import java.rmi.RemoteException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.CertificateEncodingException;
import java.security.cert.X509Certificate;
import java.util.Properties;

import org.w3._2000._09.xmldsig_.KeyInfo;
import org.w3._2000._09.xmldsig_.X509Data;

import dk.dkma.medicinecard.xml_schema._2008._06._01.MedicineCardRequestStructureType;
import dk.dkma.medicinecard.xml_schema._2008._06._01.MedicineCardResponseType;
import dk.dkma.medicinecard.xml_schema._2008._06._01.MedicineCardVersionRequestStructureType;
import dk.rsd.dcc.demo.SOSIHandler;
import dk.rsd.dcc.demo.SosiUserData;
import dk.sosi._2007._07.decoupling_component_1_0.DecouplingHeader;
import dk.sosi.gw._2007_09.LogoutRequestBody;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningRequestBody;
import dk.sosi.gw._2007_09.RequestIdCardDigestForSigningResponse;
import dk.sosi.gw._2007_09.SignIdCardRequestBody;
import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.SignatureUtil;
import dk.sosi.seal.vault.GenericCredentialVault;
import dk.sosi.www._2007._07.decoupling_component_1_0_xsd.DGWSFault;

public abstract class IntegrationTestSuite extends AbstractDccTestSuite {

	public IntegrationTestSuite(JMSHelper jmsHelper) {
		super(jmsHelper);
	}

	protected void setUp() throws Exception {
		super.setUp();
		facade._getServiceClient().engageModule("sosi");
		SOSIHandler.setUserData(new SosiUserData());
		loginIntoSosiGW();
	}

	protected void tearDown() throws Exception {
		logoutFromSosiGW();
		facade._getServiceClient().disengageModule("sosi");
		super.tearDown();
	}

	public void testSynchCall() throws Exception {
		MedicineCardRequestStructureType requestStructureType = new MedicineCardRequestStructureType();
		requestStructureType.setPersonCivilRegistrationIdentifier(CPR);

		DecouplingHeader decouplingHeader = new DecouplingHeader();
		decouplingHeader.setTimeout(5000);

		MedicineCardResponseType medicineCard = facade.GetMedicineCard(requestStructureType, null, null, decouplingHeader);
		assertEquals(CPR, medicineCard.getMedicineCardOverviewStructure().get(0).getPatientStructure().getSimpleCPRPerson().getPersonCivilRegistrationIdentifier());

		requestStructureType.setPersonCivilRegistrationIdentifier("sleep 10000");
		try {
			facade.GetMedicineCard(requestStructureType, null, null, decouplingHeader);
			fail("timeout expected");
		} catch (DGWSFault fault) {
			assertEquals("dcc_timeout", fault.getFaultMessage());
		}

		MedicineCardVersionRequestStructureType versionRequestStructureType = new MedicineCardVersionRequestStructureType();
		try {
			facade.GetMedicineCardVersion(versionRequestStructureType, null, null, decouplingHeader);
			fail("SOSIGW proxy error expected");
		} catch (DGWSFault fault) {
			assertEquals("sosigw_proxy_error", fault.getFaultMessage());
		}
	}

	public void testMissingIdcardInSosiGW() throws Exception {
		logoutFromSosiGW();
		assertNoValidIdcardInCache();
	}

	public void testTooOldIdcardInSosiGW() throws Exception {
		Thread.sleep(2 * 60 * 1000);
		assertNoValidIdcardInCache();
	}

	private void assertNoValidIdcardInCache() throws RemoteException {
		MedicineCardRequestStructureType requestStructureType = new MedicineCardRequestStructureType();
		requestStructureType.setPersonCivilRegistrationIdentifier(CPR);

		DecouplingHeader decouplingHeader = new DecouplingHeader();
		decouplingHeader.setTimeout(1000);
		try {
			facade.GetMedicineCard(requestStructureType, null, null, decouplingHeader);
			fail("'sosigw_no_valid_idcard_in_cache' fault expected");
		} catch (DGWSFault fault) {
			assertEquals("sosigw_no_valid_idcard_in_cache", fault.getFaultMessage());
		}

		try {
			facade.GetMedicineCard(requestStructureType, null, null, decouplingHeader);
			fail("'sosigw_no_valid_idcard_in_cache' fault expected");
		} catch (DGWSFault fault) {
			assertEquals("sosigw_no_valid_idcard_in_cache", fault.getFaultMessage());
		}

		try {
			facade.GetMedicineCard(requestStructureType, null, null, decouplingHeader);
			fail("'sosigw_no_valid_idcard_in_cache' fault expected");
		} catch (DGWSFault fault) {
			assertEquals("sosigw_no_valid_idcard_in_cache", fault.getFaultMessage());
		}
	}

	private void loginIntoSosiGW() throws RemoteException, DGWSFault, NoSuchAlgorithmException, NoSuchProviderException, InvalidKeyException,
	SignatureException, CertificateEncodingException {
		RequestIdCardDigestForSigningRequestBody body = new RequestIdCardDigestForSigningRequestBody();
		body.setValue("whatever");

		RequestIdCardDigestForSigningResponse signingResponse = facade.requestIdCardDigestForSigning(body, null, null);
		byte[] digestValue = signingResponse.getDigestValue();

		// As a convenience GenericCredentialVault is used to load the
		// certificate
		Properties properties = SignatureUtil.setupCryptoProviderForJVM();
		GenericCredentialVault vault = new GenericCredentialVault(properties, "!234Qwer");
		vault.setSystemCredentialPair(this.getClass().getResourceAsStream("/cvr-19343634.pfx"), "!234Qwer");

		// In a application setup, the signing would be involve user-interaction
		Signature jceSign = Signature.getInstance("SHA1withRSA", SignatureUtil.getCryptoProvider(properties,
				SOSIFactory.PROPERTYNAME_SOSI_CRYPTOPROVIDER_SHA1WITHRSA));
		PrivateKey key = vault.getSystemCredentialPair().getPrivateKey();
		jceSign.initSign(key);
		jceSign.update(digestValue);

		byte[] signedBytes = jceSign.sign();
		X509Certificate certificate = vault.getSystemCredentialPair().getCertificate();

		X509Data data = new X509Data();
		data.setX509Certificate(certificate.getEncoded());

		KeyInfo keyInfo = new KeyInfo();
		keyInfo.setX509Data(data);

		SignIdCardRequestBody signIdCardRequestBody = new SignIdCardRequestBody();
		signIdCardRequestBody.setKeyInfo(keyInfo);
		signIdCardRequestBody.setSignatureValue(signedBytes);

		facade.signIdCard(signIdCardRequestBody, null, null);
	}

	private void logoutFromSosiGW() throws RemoteException {
		LogoutRequestBody logoutRequestBody = new LogoutRequestBody();
		logoutRequestBody.setValue("whatever");
		facade.logout(logoutRequestBody, null, null);
	}

}
