package dk.rsd.dcc.client;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import junit.framework.TestCase;

public class JMeterTestClient extends TestCase {

	private static final String SOAP_ACTION = "http://www.dkma.dk/medicinecard/xml.schema/2008/06/01#GetMedicineCard";
	private static final int ITERATIONS = Integer.getInteger("dk.rsd.dcc.responsetimedivider", 1).intValue();

	private static URL url;
	private static String request;
	private static String response;

	static {
		try {
			String host = System.getProperty("dcc.host", "localhost:8080");
			url = new URL("http://" + host + "/decoupling/");
			request = readStream(TestCase.class.getResourceAsStream("/soap/jmeter-request.xml"));
			response = readStream(TestCase.class.getResourceAsStream("/soap/jmeter-response.xml"));
			System.out.println("Iterations: " + ITERATIONS);
			System.out.println("Request: " + (request.length() / 1024) + "KB");
			System.out.println("Response: " + (response.length() / 1024) + "KB");
		} catch (MalformedURLException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void testCall() throws Exception {
		for (int i = 0; i < ITERATIONS; i++) {
			InputStream is = new SoapClient().callService(url, request, SOAP_ACTION);
			assertEquals(response, readStream(is));
		}
	}

	private static String readStream(InputStream is) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		byte[] buffer = new byte[8192];
		int n;
		while ((n = is.read(buffer, 0, buffer.length)) != -1) {
			baos.write(buffer, 0, n);
		}
		return new String(baos.toByteArray());
	}

}
