/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/test/java/dk/rsd/dcc/client/MedicinecardTestClient.java $
 * $Id: MedicinecardTestClient.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.client;

import java.io.File;

import junit.framework.TestCase;
import junit.textui.TestRunner;

import org.apache.axiom.soap.SOAPFault;
import org.apache.axis2.client.Options;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import dk.dkma.medicinecard.xml_schema._2008._06._01.MedicineCardRequestStructureType;
import dk.dkma.medicinecard.xml_schema._2008._06._01.MedicineCardResponseType;
import dk.sosi._2007._07.decoupling_component_1_0.DecouplingHeader;
import dk.sosi.www._2007._07.decoupling_component_1_0_xsd.DGWSFault;
import dk.sosi.www._2007._07.decoupling_component_1_0_xsd.DccFacadeServiceStub;

public class MedicinecardTestClient extends TestCase {
	//private static final String ENDPOINT = "http://rsd.lakeside.dk/decoupling/";
	// private static final String ENDPOINT = "http://rsd.lakeside.dk/decoupling-ws/";
	//private static final String ENDPOINT = "http://localhost:8080/provider-dcc/services/MedicineCard/";
	//private static final String ENDPOINT = "http://cosmic.lms.trifork.com/ws/MedicineCard";
	//private static final String ENDPOINT = "http://trifork.lms.trifork.com/stub/ws/MedicineCard";
	//private static final String ENDPOINT = "http://trifork.lms.trifork.com/ws/MedicineCard";
	//private static final String ENDPOINT = "http://fmk-cosmic.lakeside.dk/decoupling/";
	//private static final String ENDPOINT = "http://192.168.114.128:9081/decoupling/";
	//private static final String ENDPOINT = "http://10.161.2.82:9080/decoupling/";
	//private static final String ENDPOINT = "http://localhost:8080/fi-provider/service/MedicineCard";
	//private static final String ENDPOINT = "http://fmk-cosmic.lakeside.dk/fi-provider/service/MedicineCard";
	private static final String ENDPOINT = "http://localhost:8080/decoupling/";

	//private static final String CPR = "0812926632";
	//private static final String CPR = "1111111118";
	private static final String CPR = "0207655309";

	private static Logger log = Logger.getLogger(MedicinecardTestClient.class);

	protected void setUp() throws Exception {
		super.setUp();
		PatternLayout patternLayout = new PatternLayout("%-5p %c - %m%n");
		ConsoleAppender consoleAppender = new ConsoleAppender(patternLayout);
		consoleAppender.setThreshold(Level.INFO);
		BasicConfigurator.configure(consoleAppender);
	}

	public void testCall() throws Exception {

		File file = new File("src/test/resources/axis2.xml");
		ConfigurationContext ctx = ConfigurationContextFactory.createConfigurationContextFromFileSystem(null, file.getAbsolutePath());
		DccFacadeServiceStub facade = new DccFacadeServiceStub(ctx, ENDPOINT);

		try {

			Options options = facade._getServiceClient().getOptions();
			options.setProperty(HTTPConstants.SO_TIMEOUT, new Integer(60 * 1000));
			//options.setProperty(org.apache.axis2.transport.http.HTTPConstants.MC_GZIP_REQUEST, Boolean.TRUE);
			options.setProperty(org.apache.axis2.transport.http.HTTPConstants.MC_ACCEPT_GZIP, Boolean.TRUE);
			//options.setProperty(Constants.Configuration.MESSAGE_TYPE, "application/fastinfoset");

//			CreateDrugMedicationRequestStructureType createDrugMedicationRequestStructure = new CreateDrugMedicationRequestStructureType();
//			CreateDrugMedicationStructureType type = new CreateDrugMedicationStructureType();
//			type.setDrugMedicationTypeIdentifier(DrugMedicationTypeIdentifierType.EFTER_BEHOV);
//			createDrugMedicationRequestStructure.getCreateDrugMedicationStructure().add(type);
//			facade.CreateDrugMedication(createDrugMedicationRequestStructure , null, null, null);


			MedicineCardRequestStructureType requestStructureType = new MedicineCardRequestStructureType();
			requestStructureType.setPersonCivilRegistrationIdentifier(CPR);

			DecouplingHeader decouplingHeader = new DecouplingHeader();
			decouplingHeader.setTimeout(60000);

			// With decoupling header
//			long start = System.currentTimeMillis();
//			MedicineCardResponseType medicineCard = facade.GetMedicineCard(requestStructureType, null, null, decouplingHeader);
//			System.out.println(System.currentTimeMillis() - start);
//			assertEquals(CPR, medicineCard.getMedicineCardOverviewStructure().get(0).getPatientStructure().getSimpleCPRPerson()
//					.getPersonCivilRegistrationIdentifier());

			// Without decoupling header
			requestStructureType.setIncludeNonReviewedOnly(true);
			//for (int i = 0; i < 10; i++) {
			long start = System.currentTimeMillis();
			MedicineCardResponseType medicineCard = facade.GetMedicineCard(requestStructureType, null, null, null);
			System.out.println("Time: " + (System.currentTimeMillis() - start));

			assertEquals(CPR, medicineCard.getMedicineCardOverviewStructure().get(0).getPatientStructure().getSimpleCPRPerson()
					.getPersonCivilRegistrationIdentifier());
			//}
			log.info("Succes: Test passed!");

		} catch (DGWSFault fault) {
			MessageContext messageContext = facade._getServiceClient().getLastOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
			SOAPFault soapFault = messageContext.getEnvelope().getBody().getFault();
			String faultString = soapFault.getReason().getText();
			log.error("DGWSFault from MedicineCard: " + fault.getFaultMessage() + " : " + faultString);
			throw fault;
		}

	}

	public static void main(String[] args) throws Exception {
		TestRunner.run(MedicinecardTestClient.class);
	}
}
