/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/test/java/dk/rsd/dcc/client/SystemTestSuite.java $
 * $Id: SystemTestSuite.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.client;

import org.apache.axis2.client.Options;

import dk.dkma.medicinecard.xml_schema._2008._06._01.MedicineCardRequestStructureType;
import dk.dkma.medicinecard.xml_schema._2008._06._01.MedicineCardResponseType;
import dk.dkma.medicinecard.xml_schema._2008._06._01.MedicineCardVersionRequestStructureType;
import dk.sosi._2007._07.decoupling_component_1_0.DecouplingHeader;
import dk.sosi.www._2007._07.decoupling_component_1_0_xsd.DGWSFault;

public abstract class SystemTestSuite extends AbstractDccTestSuite  {

	public SystemTestSuite(JMSHelper jmsHelper) {
		super(jmsHelper);
	}

	public void testSynchCall() throws Exception {
		MedicineCardRequestStructureType requestStructureType = new MedicineCardRequestStructureType();
		requestStructureType.setPersonCivilRegistrationIdentifier(CPR);

		DecouplingHeader decouplingHeader = new DecouplingHeader();
		decouplingHeader.setTimeout(5000);

		Options options = facade._getServiceClient().getOptions();
		options.setProperty(org.apache.axis2.transport.http.HTTPConstants.MC_ACCEPT_GZIP, Boolean.TRUE);

		MedicineCardResponseType medicineCard = facade.GetMedicineCard(requestStructureType, null, null, decouplingHeader);
		assertEquals(CPR, medicineCard.getMedicineCardOverviewStructure().get(0).getPatientStructure().getSimpleCPRPerson().getPersonCivilRegistrationIdentifier());

		requestStructureType.setPersonCivilRegistrationIdentifier("sleep 10000");
		try {
			facade.GetMedicineCard(requestStructureType, null, null, decouplingHeader);
			fail("timeout expected");
		} catch (DGWSFault fault) {
			assertEquals("dcc_timeout", fault.getFaultMessage());
		}

		MedicineCardVersionRequestStructureType versionRequestStructureType = new MedicineCardVersionRequestStructureType();
		decouplingHeader.setTimeout(30000);
		try {
			facade.GetMedicineCardVersion(versionRequestStructureType, null, null, decouplingHeader);
			fail("Expected 'dcc_general_error' fault");
		} catch (DGWSFault fault) {
			assertEquals("dcc_general_error", fault.getFaultMessage());
		}

	}

}
