/*
 *  The MIT License
 *  Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2016 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/test/java/dk/rsd/dcc/dispatch/ssl/SSLDispatcherContextTest.java $
 * $Id: SSLDispatcherContextTest.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.dispatch.ssl;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.varia.NullAppender;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import javax.net.ssl.SSLContext;
import java.security.GeneralSecurityException;
import java.security.KeyStore;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;

/**
 * Created by chg on 25/04/2016.
 */
public class SSLDispatcherContextTest {

    @BeforeClass
    public static void setupConfig() throws GeneralSecurityException {
        BasicConfigurator.configure(new NullAppender());
    }

    @AfterClass
    public static void resetConfig() {
        BasicConfigurator.resetConfiguration();
    }

    @Before
    public void resetSSLDispatcherContext() {
        SSLDispatcherContext.reset();
    }

    @Test(expected = IllegalStateException.class)
    public void testUninitalizedInstance() {
        SSLDispatcherContext.getSSLContext();
    }

    @Test
    public void testIntializedWithDefaultTrust() throws Exception {
        SSLContext defaultSSLContext = SSLContext.getDefault();

        SSLDispatcherContext.init(null);
        SSLContext dccSSLContext = SSLDispatcherContext.getSSLContext();
        assertEquals(defaultSSLContext, dccSSLContext);
    }

    @Test
    public void testIntializedWithAdditionalTrust() throws Exception {
        SSLContext defaultSSLContext = SSLContext.getDefault();

        KeyStore trustStore = KeyStore.getInstance(KeyStore.getDefaultType());
        SSLDispatcherContext.init(trustStore);
        SSLContext dccSSLContext = SSLDispatcherContext.getSSLContext();
        assertNotEquals(defaultSSLContext, dccSSLContext);
    }

}
