/*
 * The MIT License
 *
 * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Copyright (C) 2015 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/test/java/dk/rsd/dcc/fault/FaultUtilTest.java $
 * $Id: FaultUtilTest.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.fault;

import dk.rsd.dcc.DccConfig;
import dk.rsd.dcc.DccConstants;
import dk.rsd.dcc.DccContext;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.varia.NullAppender;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.net.URL;

import static org.junit.Assert.assertEquals;

public class FaultUtilTest {

    @BeforeClass
    public static void loadDccConfig() {
        BasicConfigurator.configure(new NullAppender());
        System.setProperty(DccConstants.PROPERTY_DCC_CONFIGFILE, "config/dcc-config-test.xml");
        DccConfig.getInstance();
    }

    @AfterClass
    public static void reloadDccConfig() {
        DccConfig.reload();
        BasicConfigurator.resetConfiguration();
    }

    @Test
    public void testFormatDestination() throws Exception {
        DccContext context = new DccContext();
        assertEquals("<UNKNOWN> for operation 'null'", FaultUtil.formatDestination(context, false));
        assertEquals("<UNKNOWN> for operation 'null'", FaultUtil.formatDestination(context, true));

        context.setSoapAction("http://www.example-1.org#doIt");
        assertEquals("<UNKNOWN> for operation 'http://www.example-1.org#doIt'", FaultUtil.formatDestination(context, false));
        assertEquals("<UNKNOWN> for operation 'http://www.example-1.org#doIt'", FaultUtil.formatDestination(context, true));

        context.setServiceURL(new URL("http://foo:8080/bar/xyz?foo=bar#zzz"));
        assertEquals("http://foo:8080/bar/xyz?foo=bar#zzz for operation 'http://www.example-1.org#doIt'", FaultUtil.formatDestination(context, false));
        assertEquals("http://<MASKED>/bar/xyz?foo=bar#zzz for operation 'http://www.example-1.org#doIt'", FaultUtil.formatDestination(context, true));

        context.setProxyURL(new URL("https://example.com/xyz?bar=1&foo=2#zzz"));
        assertEquals("http://foo:8080/bar/xyz?foo=bar#zzz through proxy https://example.com/xyz?bar=1&foo=2#zzz for operation 'http://www.example-1.org#doIt'",
                FaultUtil.formatDestination(context, false));
        assertEquals("http://<MASKED>/bar/xyz?foo=bar#zzz through proxy https://<MASKED>/xyz?bar=1&foo=2#zzz for operation 'http://www.example-1.org#doIt'",
                FaultUtil.formatDestination(context, true));
    }
}
