/*
 *
 *  * The MIT License
 *  *
 *  * Original work sponsored and donated by National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *  *
 *  * Copyright (C) 2014 National Board of e-Health (NSI), Denmark (http://www.nsi.dk)
 *  *
 *  * Permission is hereby granted, free of charge, to any person obtaining a copy of
 *  * this software and associated documentation files (the "Software"), to deal in
 *  * the Software without restriction, including without limitation the rights to
 *  * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 *  * of the Software, and to permit persons to whom the Software is furnished to do
 *  * so, subject to the following conditions:
 *  *
 *  * The above copyright notice and this permission notice shall be included in all
 *  * copies or substantial portions of the Software.
 *  *
 *  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  * SOFTWARE.
 *  *
 *  * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/dcc/src/test/java/dk/rsd/dcc/routing/DatabaseTestUtils.java $
 *  * $Id: DatabaseTestUtils.java 33 2017-01-12 16:52:05Z chg $
 *
 */

package dk.rsd.dcc.routing;

import org.h2.jdbcx.JdbcDataSource;
import org.springframework.jdbc.core.JdbcTemplate;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.util.Date;

/**
 * Created by chg on 08/04/2014.
 */
public class DatabaseTestUtils {

    public static Context initAndBindDataSource(JdbcDataSource ds) throws NamingException {
        System.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.apache.naming.java.javaURLContextFactory");
        System.setProperty(Context.URL_PKG_PREFIXES, "org.apache.naming");
        Context ctx = new InitialContext();

        ds.setURL("jdbc:h2:mem:dcc_test;MODE=MYSQL;DB_CLOSE_DELAY=-1");
        ctx.createSubcontext("jdbc");
        ctx.bind("jdbc/dcc_test", ds);
        return ctx;
    }

    public static void unBindDataSource(Context ctx) throws NamingException {
        ctx.unbind("jdbc");
    }

    public static void populateDatabase(JdbcTemplate jdbcTemplate) {
        jdbcTemplate.execute("create table DCC_ACTIONS (SOAP_ACTION varchar(4096), SERVICE_IDENTIFIER varchar(4096), SERVICE_URL varchar(4096), TIMEOUT_MILLIS bigint(20), IDCARD_MAXAGE_MINS int(11), USE_PROXY boolean, DO_NOT_VERIFY_SSL_HOSTNAME boolean, WSA_HEADERS_PROCESSING varchar(256) )");
        jdbcTemplate.update("insert into DCC_ACTIONS (SOAP_ACTION, SERVICE_URL, TIMEOUT_MILLIS, IDCARD_MAXAGE_MINS, USE_PROXY, DO_NOT_VERIFY_SSL_HOSTNAME, WSA_HEADERS_PROCESSING) values (?,?,?,?,?,?,?)",
                "http://foo.bar#doIt", "http://service.url", 10000, 60, true, false, "WSA_OVERWRITE");

        jdbcTemplate.update("insert into DCC_ACTIONS (SOAP_ACTION, SERVICE_URL, TIMEOUT_MILLIS, IDCARD_MAXAGE_MINS, USE_PROXY, DO_NOT_VERIFY_SSL_HOSTNAME, WSA_HEADERS_PROCESSING) values (?,?,?,?,?,?,?)",
                "http://foo.bar#doItAgain", "http://another.service.url", 5000, null, false, true, "WSA_REMOVE");

        jdbcTemplate.update("insert into DCC_ACTIONS (SOAP_ACTION, SERVICE_URL, TIMEOUT_MILLIS, IDCARD_MAXAGE_MINS, USE_PROXY, DO_NOT_VERIFY_SSL_HOSTNAME, WSA_HEADERS_PROCESSING) values (?,?,?,?,?,?,?)",
                "http://foo.bar#doItOneMoreTime", "http://another.service.url", 5000, 60, false, null, "WSA_PASSTHROUGH");

        jdbcTemplate.update("insert into DCC_ACTIONS (SOAP_ACTION, SERVICE_URL, TIMEOUT_MILLIS, IDCARD_MAXAGE_MINS, USE_PROXY, DO_NOT_VERIFY_SSL_HOSTNAME, WSA_HEADERS_PROCESSING) values (?,?,?,?,?,?,?)",
                "http://foo.bar#doItYetOneMoreTime", "http://another.service.url", 5000, 60, false, null, null);

        jdbcTemplate.update("insert into DCC_ACTIONS (SOAP_ACTION, SERVICE_IDENTIFIER, SERVICE_URL, TIMEOUT_MILLIS, IDCARD_MAXAGE_MINS, USE_PROXY, DO_NOT_VERIFY_SSL_HOSTNAME, WSA_HEADERS_PROCESSING) values (?,?,?,?,?,?,?,?)",
                "http://foo.bar#doItWithAServiceIdentifier", "service-identifier-1", "http://another.service.url", 5000, 60, false, null, null);
    }

    public static void insertMalformedWSAHeaderProcessing(JdbcTemplate jdbcTemplate) {
        jdbcTemplate.update("insert into DCC_ACTIONS (SOAP_ACTION, SERVICE_URL, TIMEOUT_MILLIS, IDCARD_MAXAGE_MINS, USE_PROXY, DO_NOT_VERIFY_SSL_HOSTNAME, WSA_HEADERS_PROCESSING) values (?,?,?,?,?,?,?)",
                "http://foo.bar#doItTomorrow", "http://another.service.url", 5000, 60, false, null, "FOO");
    }


        public static void resetDatabase(JdbcTemplate jdbcTemplate) {
        jdbcTemplate.execute("drop all objects");
    }

}
