/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/sosi-module/src/main/java/dk/rsd/dcc/demo/Axis2Util.java $
 * $Id: Axis2Util.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.demo;

import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNode;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.util.XMLUtils;
import org.w3c.dom.Document;

import dk.sosi.seal.model.Message;
import dk.sosi.seal.model.constants.NameSpaces;

public class Axis2Util {

    /**
     * @param doc
     * @param messageContext
     * @throws Exception
     */
    private static void addHeadersToSOAPEnvelope(SOAPEnvelope envelope, Document doc) throws Exception {
        
    	// Make sure that all namespaces are present in the axis response
        Map namespaces = NameSpaces.SOSI_NAMESPACES;

        Iterator it = namespaces.keySet().iterator();
        while (it.hasNext()) {
            String namespace = (String) it.next();
            envelope.declareNamespace((String) namespaces.get(namespace), namespace);
        }

        envelope.addAttribute("id", "Envelope", null);

        // Now transfer the generated header elements back to the axis context
        OMElement omDoc = XMLUtils.toOM(doc.getDocumentElement());
        Iterator childElements = omDoc.getFirstChildWithName(new QName(NameSpaces.SOAP_SCHEMA, "Header")).getChildElements();
        while (childElements.hasNext()) {
        	envelope.getHeader().addChild((OMNode) childElements.next());
        }
        
    }

    public static void addMessageToSOAPEnvelope(MessageContext messageContext, Message message) throws Exception {
    	SOAPEnvelope envelope = messageContext.getEnvelope();
        Document doc = message.serialize2DOMDocument();
		addHeadersToSOAPEnvelope(envelope, doc);
    }

}
