/*
 * SOSI-DCC - Copyright (C) 2011 National Board of e-Health (NSI)
 *
 * All source code and information supplied as part of SOSI-DCC is
 * copyright to National Board of e-Health.
 *
 * The source code has been released under a dual license - meaning you can
 * use either licensed version of the library with your code.
 *
 * It is released under the Common Public License 1.0, a copy of which can
 * be found at the link below.
 * http://www.opensource.org/licenses/cpl1.0.php
 *
 * It is released under the LGPL (GNU Lesser General Public License), either
 * version 2.1 of the License, or (at your option) any later version. A copy
 * of which can be found at the link below.
 * http://www.gnu.org/copyleft/lesser.html
 *
 * $HeadURL: https://svn.nspop.dk/svn/lakeside/sosi-dcc/tags/release-2.4.3/modules/sosi-module/src/main/java/dk/rsd/dcc/demo/SOSIHandler.java $
 * $Id: SOSIHandler.java 33 2017-01-12 16:52:05Z chg $
 */
package dk.rsd.dcc.demo;

import dk.sosi.seal.SOSIFactory;
import dk.sosi.seal.model.*;
import dk.sosi.seal.model.constants.DGWSConstants;
import dk.sosi.seal.model.constants.SubjectIdentifierTypeValues;
import dk.sosi.seal.vault.EmptyCredentialVault;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.handlers.AbstractHandler;
import org.apache.axis2.wsdl.WSDLConstants;

import java.security.cert.X509Certificate;
import java.util.Properties;

public class SOSIHandler extends AbstractHandler {

	private static ThreadLocal<SosiUserData> userDataStore = new ThreadLocal<SosiUserData>();
	private static ThreadLocal<String> lastMessageID = new ThreadLocal<String>();

	public static void setUserData(SosiUserData userData) {
		userDataStore.set(userData);
		userDataStore.set(userData);
	}

	public static String getLastMessageID() {
		return lastMessageID.get();
	}

	public InvocationResponse invoke(MessageContext messagecontext) throws AxisFault {

		Properties properties = new Properties();
		properties.put(SOSIFactory.PROPERTYNAME_SOSI_DGWS_VERSION, DGWSConstants.VERSION_1_0_1);
		SOSIFactory factory = new SOSIFactory(new EmptyCredentialVault(), properties);

		SosiUserData userdata = userDataStore.get();
		CareProvider careProvider = new CareProvider(SubjectIdentifierTypeValues.CVR_NUMBER, userdata.getOrgCvr(), userdata.getOrgName());
		AuthenticationLevel authenticationLevel = AuthenticationLevel.NO_AUTHENTICATION;
		X509Certificate certificate = null;


        UserInfo userInfo = new UserInfo(userdata.getCpr(), userdata.getGivenName(), userdata.getSurName(), userdata.getEmail(), userdata.getOccupation(),
                                         userdata.getRole(), userdata.getAuthorizationCode());

        IDCard idCard = factory.createNewUserIDCard(userdata.getItSystemName(), userInfo, careProvider, authenticationLevel, null, null, certificate, null);

		// Create Request
		Request request = factory.createNewRequest(false, null);
		request.setIDCard(idCard);

		try {
			MessageContext mc = messagecontext.getOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_OUT_VALUE);
			Axis2Util.addMessageToSOAPEnvelope(mc, request);
			lastMessageID.set(request.getMessageID());
		} catch (Throwable e) {
			lastMessageID.set(null);
			throw new AxisFault(e.getMessage(), e);
		}
		return InvocationResponse.CONTINUE;
	}

}
